# -*- coding: utf-8 -*-
# Extensions/Install.py
__author__    = 'Fco. Jose Mulero - GMV'
__docformat__ = 'plaintext'
__version__   = '$id$'[11:-2]

import os.path
import sys
from StringIO import StringIO

from App.Common import package_home
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import manage_addTool
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from zExceptions import NotFound, BadRequest
from Products.Notifica.Extensions import install_utils
from Products.Notifica.Extensions import AppInstall
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
try:
    from Products.Archetypes.lib.register import listTypes
except ImportError:
    from Products.Archetypes.public import listTypes
from Products.Notifica.config import PROJECTNAME,PROPERTIES,SUBSCRIBERS_PATH, NOTIFICA_PATH, PROJECT_TYPES
from Products.Notifica.config import product_globals as GLOBALS



def install(self):
    """ External Method to install Notifica """
    out = StringIO()
    print >> out, "Installation log of %s:" % PROJECTNAME

    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    try:
        from Products.Notifica.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        get_transaction().commit(1)

    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)


    # try to call a workflow install method
    # in 'InstallWorkflows.py' method 'installWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'installWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Install:'
        res = installWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow install'

    from Products.Notifica.config import STYLESHEETS
    try:
        portal_css = getToolByName(portal, 'portal_css')
        for stylesheet in STYLESHEETS:
            try:
                portal_css.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': '',
            'media': 'all',
            'enabled': True}
            defaults.update(stylesheet)
            portal_css.manage_addStylesheet(**defaults)
            print >>out, "CSS installed"
    except:
        # No portal_css registry
        pass
    from Products.Notifica.config import JAVASCRIPTS
    try:
        portal_javascripts = getToolByName(portal, 'portal_javascripts')
        for javascript in JAVASCRIPTS:
            try:
                portal_javascripts.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': ''}
            defaults.update(javascript)
            portal_javascripts.registerScript(**defaults)
            print >>out, "JAVASCRIPT installed"
    except:
        # No portal_javascripts registry
        pass

    # try to call a custom install method
    # in 'AppInstall.py' method 'install'
    try:
        install = AppInstall.install(self,out)

    except NotFound:
        install = None

    # Install property sheet
    portal_properties = getToolByName(self, "portal_properties")

    try:
        portal_properties.addPropertySheet(
            id    = PROPERTIES,
            title = "Notifica Properties")
    except:
        pass

    # Add property
    enp = portal_properties.notifica_properties
    if not enp.hasProperty(SUBSCRIBERS_PATH):
        enp.manage_addProperty(
            id = SUBSCRIBERS_PATH,
            value = "",
            type = "string")
    if not enp.hasProperty(NOTIFICA_PATH):
        enp.manage_addProperty(
            id = NOTIFICA_PATH,
            value = "",
            type = "string")
    print >>out, "Properties installed"

    # try to install portlet
    install_utils.installPortlet(self,out)

    # try to install configlet
    install_utils.installConfiglet(self,out)

    # Enable portal_factory
    try:
        install_utils.installPortalFactory(self,out);
    except:
        print >> out, "Problem occurs when Portal Factory was been enabling"

    #try to install tool
    if not hasattr(self, 'notifica_tool'):
        addTool = self.manage_addProduct[PROJECTNAME].manage_addTool
        # Add the tool by its meta_type
        install = addTool('NotificaTool')

    if install:
        print >>out,'Custom Install:'
        res = install(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom install'
    return out.getvalue()

def uninstall(self):
    out = StringIO()

    # try to call a workflow uninstall method
    # in 'InstallWorkflows.py' method 'uninstallWorkflows'

    # TODO: this is buggy code. There is no workflow uninstaller in
    # the generated InstallWorkflows.py.
    try:
        uninstallWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'uninstallWorkflows').__of__(self)
    except NotFound:
        uninstallWorkflows = None

    if uninstallWorkflows:
        print >>out, 'Workflow Uninstall:'
        res = uninstallWorkflows(self, out)
        print >>out, res or 'no output'
    else:
        print >>out,'no workflow uninstall'

    # try to call a custom uninstall method
    # in 'AppInstall.py' method 'uninstall'
    try:
        uninstall = ExternalMethod('temp','temp',PROJECTNAME+'.AppInstall', 'uninstall')
    except:
        uninstall = None

    # Install property sheet
    portal_properties = getToolByName(self, "portal_properties")

    try:
        portal_properties.manage_delObjects([PROPERTIES])
    except:
        pass
    # try to uninstall portlet
    install_utils.uninstallPortlet(self,out)

    # try to uninstall configlet
    install_utils.uninstallConfiglet(self,out)

    # try to uninstall from portal_factory
    install_utils.uninstallPortalFactory(self,out)

    if uninstall:
        print >>out,'Custom Uninstall:'
        res = uninstall(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom uninstall'

    return out.getvalue()