# -*- coding: utf-8 -*-
#
# Notifica
# Copyright (C) 2007 GMV SGI Team <http://www.gmv-sgi.es>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# $Id: NotifNewsletter_Workflow.py 615 2007-11-28 11:40:43Z frms $
#
##Workflow: Newsletter_Workflow

__author__    = 'Fco. Jose Mulero - GMV'
__docformat__ = 'plaintext'

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from Products.Notifica.config import PROJECTNAME

##code-section create-workflow-module-header #fill in your manual code here
##/code-section create-workflow-module-header


productname = PROJECTNAME

def setupNewsletter_Workflow(self, workflow):
    """Define the Newsletter_Workflow workflow.
    """

    workflow.setProperties(title='NotifNewsletter_Workflow')

    ##code-section create-workflow-setup-method-header #fill in your manual code here
    ##/code-section create-workflow-setup-method-header


    for s in ['sent', 'draft']:
        workflow.states.addState(s)

    for t in ['make sent', 'make draft']:
        workflow.transitions.addTransition(t)

    for v in ['review_history', 'comments', 'time', 'actor', 'action']:
        workflow.variables.addVariable(v)

    for p in ['View', 'Modify portal content', 'Access contents information']:
        workflow.addManagedPermission(p)

    for l in []:
        if not l in workflow.worklists.objectValues():
            workflow.worklists.addWorklist(l)

    ## Initial State

    workflow.states.setInitialState('draft')

    ## States initialization

    stateDef = workflow.states['sent']
    stateDef.setProperties(title="""Sent""",
                           transitions=['make draft'])
    stateDef.setPermission('View',
                           0,
                           ['Manager', 'Member', 'Anonymous'])
    stateDef.setPermission('Modify portal content',
                           0,
                           ['Manager','Member'])
    stateDef.setPermission('Access contents information',
                           0,
                           ['Manager', 'Member', 'Anonymous'])

    stateDef = workflow.states['draft']
    stateDef.setProperties(title="""draft""",
                           transitions=['make sent'])
    stateDef.setPermission('View',
                           0,
                           ['Manager','Owner'])
    stateDef.setPermission('Modify portal content',
                           0,
                           ['Manager','Owner'])
    stateDef.setPermission('Access contents information',
                           0,
                           ['Manager','Owner'])

    ## Transitions initialization

    transitionDef = workflow.transitions['make sent']
    transitionDef.setProperties(title="""make sent""",
                                new_state_id="""sent""",
                                trigger_type=1,
                                script_name="""""",
                                after_script_name="""""",
                                actbox_name="""make sent""",
                                actbox_url="""""",
                                actbox_category="""workflow""",
                                props={'guard_permissions': 'Modify portal content'},
                                )

    transitionDef = workflow.transitions['make draft']
    transitionDef.setProperties(title="""make draft""",
                                new_state_id="""draft""",
                                trigger_type=1,
                                script_name="""""",
                                after_script_name="""""",
                                actbox_name="""make draft""",
                                actbox_url="""""",
                                actbox_category="""workflow""",
                                props={'guard_permissions': 'Modify portal content'},
                                )

    ## State Variable
    workflow.variables.setStateVar('review_state')

    ## Variables initialization
    variableDef = workflow.variables['review_history']
    variableDef.setProperties(description="""Provides access to workflow history""",
                              default_value="""""",
                              default_expr="""state_change/getHistory""",
                              for_catalog=0,
                              for_status=0,
                              update_always=0,
                              props={'guard_permissions': 'Request review; Review portal content'})

    variableDef = workflow.variables['comments']
    variableDef.setProperties(description="""Comments about the last transition""",
                              default_value="""""",
                              default_expr="""python:state_change.kwargs.get('comment', '')""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['time']
    variableDef.setProperties(description="""Time of the last transition""",
                              default_value="""""",
                              default_expr="""state_change/getDateTime""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['actor']
    variableDef.setProperties(description="""The ID of the user who performed the last transition""",
                              default_value="""""",
                              default_expr="""user/getId""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['action']
    variableDef.setProperties(description="""The last transition""",
                              default_value="""""",
                              default_expr="""transition/getId|nothing""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    ## Worklists Initialization


    # WARNING: below protected section is deprecated.
    # Add a tagged value 'worklist' with the worklist name to your state(s) instead.

    ##code-section create-workflow-setup-method-footer #fill in your manual code here
    ##/code-section create-workflow-setup-method-footer



def createNewsletter_Workflow(self, id):
    """Create the workflow for Notifica.
    """

    ob = DCWorkflowDefinition(id)
    setupNewsletter_Workflow(self, ob)
    return ob

addWorkflowFactory(createNewsletter_Workflow,
                   id='NotifNewsletter_Workflow',
                   title='NotifNewsletter_Workflow')

##code-section create-workflow-module-footer #fill in your manual code here
##/code-section create-workflow-module-footer

def setupSubscriber_Workflow(self, workflow):
    """Define the Newsletter_Workflow workflow.
    """

    workflow.setProperties(title='NotifSubscriber_Workflow')

    ##code-section create-workflow-setup-method-header #fill in your manual code here
    ##/code-section create-workflow-setup-method-header


    for s in ['private']:
        workflow.states.addState(s)

    for v in ['review_history', 'comments', 'time', 'actor', 'action']:
        workflow.variables.addVariable(v)

    for p in ['View', 'Modify portal content', 'Access contents information']:
        workflow.addManagedPermission(p)

    for l in []:
        if not l in workflow.worklists.objectValues():
            workflow.worklists.addWorklist(l)

    ## Initial State

    workflow.states.setInitialState('private')

    ## States initialization

    stateDef = workflow.states['private']
    stateDef.setProperties(title="""Private""")
    stateDef.setPermission('View',
                           0,
                           ['Manager','Member' ])
    stateDef.setPermission('Modify portal content',
                           0,
                           ['Manager'])
    stateDef.setPermission('Access contents information',
                           0,
                           ['Manager', 'Member' ])

    stateDef = workflow.states['private']

    ## State Variable
    workflow.variables.setStateVar('review_state')

    ## Variables initialization
    variableDef = workflow.variables['review_history']
    variableDef.setProperties(description="""Provides access to workflow history""",
                              default_value="""""",
                              default_expr="""state_change/getHistory""",
                              for_catalog=0,
                              for_status=0,
                              update_always=0,
                              props={'guard_permissions': 'Request review; Review portal content'})

    variableDef = workflow.variables['comments']
    variableDef.setProperties(description="""Comments about the last transition""",
                              default_value="""""",
                              default_expr="""python:state_change.kwargs.get('comment', '')""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['time']
    variableDef.setProperties(description="""Time of the last transition""",
                              default_value="""""",
                              default_expr="""state_change/getDateTime""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['actor']
    variableDef.setProperties(description="""The ID of the user who performed the last transition""",
                              default_value="""""",
                              default_expr="""user/getId""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['action']
    variableDef.setProperties(description="""The last transition""",
                              default_value="""""",
                              default_expr="""transition/getId|nothing""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

def createSubscriber_Workflow(self, id):
    """Create the workflow for Notifica.
    """

    ob = DCWorkflowDefinition(id)
    setupSubscriber_Workflow(self, ob)
    return ob

    addWorkflowFactory(createSubscriber_Workflow,
                   id='NotifSubscriber_Workflow',
                   title='NotifSubscriber_Workflow')


    # WARNING: below protected section is deprecated.
    # Add a tagged value 'worklist' with the worklist name to your state(s) instead.

    ##code-section create-workflow-setup-method-footer #fill in your manual code here
    ##/code-section create-workflow-setup-method-footer
