# -*- coding: utf-8 -*-
#
# Notifica
# Copyright (C) 2007 GMV SGI Team <http://www.gmv-sgi.es>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# $Id: NotificaTool.py 615 2007-11-28 11:40:43Z frms $
#
import time
import logging
import re
import os


from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager

from Products.CMFCore import permissions
from Products.CMFCore.utils import UniqueObject,getToolByName

from Products.Notifica.config import PROJECTNAME,SUBSCRIBERS_PATH,PROPERTIES

from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email import Encoders
import zlib

logger = logging.getLogger(PROJECTNAME)

SUBSCRIBE = 'subscribe'
UNSUBSCRIBE = 'unsubscribe'

class NotificaTool(UniqueObject, SimpleItem):
    """
        Tool for subscription functionality
    """
    id = 'notifica_tool'
    meta_type = 'NotificaTool'
    plone_tool = 1
    title = "Notifica Tool"

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(permissions.ManagePortal, 'overview')

    security.declarePublic('subscribe')
    def subscribe(self, subscriber):
        """Validates the email address and if it valid
           send a confirmation email for the subscription
           """
        from Products.validation.validators.BaseValidators import EMAIL_RE

        EMAIL_RE = "^" + EMAIL_RE
        mo = re.search(EMAIL_RE, subscriber)

        if mo is None:
            return (False, 1)
        else:
            try:
                subs = self.getSubscriber(subscriber)
                logger.info('Subscribe getSubscriber(%s) = %s'%(subscriber,subs))
                if(not subs is None):
                    logger.info('Subscriber %s alredry registered'%subscriber)
                    return (False,2)

                self.sendNotificationMail(subscriber,SUBSCRIBE)
                logger.info('Subscription mail sended to: %s' %subscriber)
                return (True, 0)
            except:
                logger.error('Subscription mail sended to: %s failure' %subscriber)
                return (False,1)


    security.declarePublic('unsubscribe')
    def unsubscribe(self, subscriber):
        """Validates the email address and if it valid
           send a confirmation email to unsubscribe the NotifSubscriber.
        """
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()

        from Products.validation.validators.BaseValidators import EMAIL_RE

        EMAIL_RE = "^" + EMAIL_RE
        mo = re.search(EMAIL_RE, subscriber)

        if mo is None:
            return (False, 2)
        else:

            container = self.getContainer()

            try:
                subs = self.getSubscriber(subscriber)
                if(subs is None):
                    logger.info('The subscriber %s not exists' %subscriber)
                    return (False,1)
            except:
                logger.info('Error at getSubscriber')
                return (False,1)

            try:
                self.sendNotificationMail(subscriber,UNSUBSCRIBE)
                logger.info('Unsubscription mail sended to: %s' %subscriber)
                return (True, 0)
            except:
                logger.error('Unsubscription mail sended to: %s failure' %subscriber)
                return (False, 2)

    security.declarePublic('addSubscriber')
    def addSubscriber(self,code):
        """Add a subscriber to the indicated content folder in the property subscriber_path"""
        ## Decode the subscriber code
        subscriber = self.decode(code)

        from Products.validation.validators.BaseValidators import EMAIL_RE
        EMAIL_RE = "^" + EMAIL_RE
        mo = re.search(EMAIL_RE, subscriber)

        if mo is None:
            logger.info('The email: %s is incorrect' % subscriber)
            valid_email,error_code = (False, 1)
        else:
            # Normalize Subscriber
            user = self.getWrappedOwner()
            newSecurityManager(None,user)

            plone_tool = getToolByName(self, 'plone_utils')
            subscriber_id = plone_tool.normalizeString(subscriber)

            container = self.getContainer()


            try:

                container.invokeFactory('NotifSubscriber',subscriber_id)
                noSecurityManager()

                logger.info('The subscriber: %s has been added' % subscriber)
                valid_email,error_code=(True, 0)
            except:
                noSecurityManager()
                logger.error('The email: %s already exits,the subscriber hasnt been added' % subscriber)
                valid_email,error_code = (False,2)

            if(valid_email):
                o = getattr(container, subscriber_id)
                o.setEmail(subscriber)

        return (valid_email,error_code)


    security.declarePublic('deleteSubscriber')
    def deleteSubscriber(self, code):
        """Remove a subscriber from the indicated content folder in the property subscriber_path"""
        subscriber = self.decode(code)

        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()

        user = self.getWrappedOwner()
        newSecurityManager(None,user)
        o = self.getSubscriber(subscriber)

        try:
            logger.info("%s content folder %s" %(subscriber,o.getParentNode().absolute_url))
            contenedor = o.getParentNode()
            contenedor.manage_delObjects([o.id])
            noSecurityManager()
            logger.info('The subscriber: %s has been deleted' % subscriber)
            return (True,0)
        except:
            noSecurityManager()
            logger.error('The subscriber: %s not exists' % subscriber)
            return (False,1)

    security.declarePrivate("sendNotificationMail")
    def sendNotificationMail(self,subscriber,operation):
        """Private method used to send notification mails"""
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()


        props = getToolByName(self, "portal_properties").site_properties
        encoding = props.getProperty("default_charset")
        #Get the templates for subscription / unsubscription mails
        if(operation == UNSUBSCRIBE):
            template = getattr(self, 'unsubscribe_template')
            cmd="%s/delete_subscriber?code=%s" % (portal.absolute_url(),self.encode(subscriber))
            message = template(self,unsubscribe_cmd=cmd)
            subject = self.translate('mail_unsubscription_subject',domain='notifica')
        elif (operation == SUBSCRIBE):
            template = getattr(self, 'subscribe_template')
            cmd="%s/add_subscriber?code=%s" % (portal.absolute_url(),self.encode(subscriber))
            message = template(self,subscribe_cmd=cmd)
            subject = self.translate('mail_subscription_subject',domain='notifica')

        #The senderName and the senderEmail are extracted from the portal properties
        senderEmail = portal.getProperty('email_from_address')
        senderName = portal.getProperty('email_from_name')

        mail = MIMEMultipart("alternative")
        mail['From']    = "%s <%s>" %(senderName,senderEmail)
        mail['To']      = subscriber
        mail['Subject'] = subject
        text_part = MIMEText(message, "plain", encoding)
        mail.attach(text_part)

        result = self.MailHost.send(mail.as_string())

    security.declarePrivate("getContainer")
    def getContainer(self):
        """Private Method to get the content folder for the subscribers"""
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()

        propertiesTool = getToolByName(self, 'portal_properties')
        contenedorName = getattr(getattr(propertiesTool, PROPERTIES), SUBSCRIBERS_PATH )

        contenedor=portal
        if(contenedorName is not None and contenedorName != ''):
            for s in contenedorName.split('/'):
                contenedor =getattr(contenedor,s)

        return contenedor

    security.declarePrivate("getSubscriber")
    def getSubscriber(self,mail):
        """get a susbscriber object by its mail"""
        container = self.getContainer()
        logger.info("getSuscriber objects %s" %container.contentValues())
        for o in container.contentValues():
            try:
                logger.info("getSubscriber mail of: %s" %o)
                if mail == o.getEmail():
                    return o
            except:
                pass
        return None

    security.declarePrivate("encode")
    def encode(self,text):
        """Encode the a text"""
        logger.info("Encoding: %s" % text)
        code = ''
        try:
            pre_code = zlib.compress(text)
            code = pre_code.encode("hex")

        except:
            logger.error("Encoding error")

        return code

    security.declarePrivate("decode")
    def decode(self,code):
        """Decode the a text"""
        logger.info("Decoding: %s" % code)
        text = ''
        try:
            pre_text = code.decode("hex")
            text = zlib.decompress(pre_text)

        except:
            logger.error("Decoding error")

        return text

    security.declareProtected(permissions.ManagePortal,'getSubscriberContainerPath')
    def getSubscriberContainerPath(self):
        """Public Method to get the path to content folder for the subscribers"""
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()

        propertiesTool = getToolByName(self, 'portal_properties')
        contenedorName = getattr(getattr(propertiesTool, PROPERTIES), SUBSCRIBERS_PATH )

        return contenedorName

    security.declareProtected(permissions.ManagePortal,'modifyProperties')
    def modifyProperties(self,value):
        logger.info("modifyProperties %s" % value)
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()

        portal_properties = getToolByName(self, 'portal_properties')
        try:
            if not(value is None) and value!='':
                contenedor=portal
                for s in value.split('/'):
                    contenedor = getattr(contenedor,s)
            property = portal_properties.notifica_properties
            property.manage_changeProperties({SUBSCRIBERS_PATH:value})
            return True
        except:
            logger.info("ModifyProperties: The path %s doesn't exists"%value)
            return False

InitializeClass(NotificaTool)