# -*- coding: utf-8 -*-
#
# Notifica
# Copyright (C) 2007 GMV SGI Team <http://www.gmv-sgi.es>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# $Id: __init__.py 615 2007-11-28 11:40:43Z frms $
#
# Initialise the product's module. There are three ways to inject custom code
# here:
#
#   - To set global configuration variables, create a file AppConfig.py. This
#       will be imported in config.py, which in turn is imported in each
#       generated class and in this file.
#   - To perform custom initialisation after types have been registered, use
#       the protected code section at the bottom of initialize().
#   - To register a customisation policy, create a file CustomizationPolicy.py
#       with a method register(context) to register the policy
#

from zLOG import LOG, INFO

LOG('Notifica',INFO, 'Installing Product')

try:
    import CustomizationPolicy
except ImportError:
    CustomizationPolicy=None

from Globals import package_home
from AccessControl import allow_module
from Products.CMFCore import utils as cmfutils
from Products.CMFCore import permissions as CMFCorePermissions
from Products.CMFCore import DirectoryView
from Products.CMFPlone.PloneUtilities import ToolInit
from Products.Archetypes.public import *
from Products.Archetypes import listTypes,process_types
from Products.Archetypes.utils import capitalize
from config import PROJECTNAME, product_globals
from permissions import ADD_CONTENT_PERMISSIONS
import os, os.path


DirectoryView.registerDirectory('skins', product_globals)
DirectoryView.registerDirectory('skins/Notifica',
                                    product_globals)

allow_module('Products.validation.validation')

##code-section custom-init-head #fill in your manual code here
##/code-section custom-init-head



def initialize(context):
    ##code-section custom-init-top #fill in your manual code here
    ##/code-section custom-init-top

    # imports packages and types for registration
    import interfaces
    import content
    from NotificaTool import NotificaTool

    # initialize portal content
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.portal_type)
        cmfutils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = ADD_CONTENT_PERMISSIONS[atype.portal_type],
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)

    # apply customization-policy, if theres any
    if CustomizationPolicy and hasattr(CustomizationPolicy, 'register'):
        CustomizationPolicy.register(context)
        print 'Customization policy for Notifica installed'


    tools = ( NotificaTool, )

    cmfutils.ToolInit('%s Tool' % PROJECTNAME, tools=tools,
                       product_name=PROJECTNAME, icon='tool.gif',
                       ).initialize(context)
    ##code-section custom-init-bottom #fill in your manual code here
    ##/code-section custom-init-bottom

