# -*- coding: utf-8 -*-
#
# Notifica
# Copyright (C) 2007 GMV SGI Team <http://www.gmv-sgi.es>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# $Id: NotifNewsletter.py 615 2007-11-28 11:40:43Z frms $
# File: NotifNewsletter

__author__  = 'Fco. Jose Mulero - GMV'
__docformat__ = 'plaintext'

import logging
from zope import interface
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from NotifBase import NotifBase
from Products.Notifica.interfaces.INotifNewsletter import INotifNewsletter
from Products.CMFCore import permissions as CMFCorePermissions

from Products.Notifica.config import *
##code-section module-header #fill in your manual code here
from urlparse import urlparse
from email.base64MIME import header_encode
from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email import Encoders
from Products.ATContentTypes.content.base import updateAliases
from Products.Notifica.utils.ENLHTMLParser import ENLHTMLParser

from Products.CMFCore.utils import getToolByName

import re
from Products.validation.validators.BaseValidators import EMAIL_RE

##/code-section module-header

logger = logging.getLogger(PROJECTNAME)

schema=Schema((

    StringField('notifTemplate',
        default='default',
        widget=StringWidget(
            macro="NewsletterTemplateWidget",
            label="Newsletter Template",
            description="Template, to generate the newsletter.",
            label_msgid='notifica_label_template',
            description_msgid='notifica_help_template',
            i18n_domain='notifica',
        ),
        required=1
    ),

    TextField('text',
        allowable_content_types=('text/plain', 'text/structured', 'text/html', 'application/msword',),
        widget=RichWidget(
            label='Text',
            rows=25,
            label_msgid='notifica_label_text',
            description_msgid='notifica_help_text',
            i18n_domain='notifica',
        ),
        default_output_type='text/html'

    ),


),
)


##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

Newsletter_schema = BaseSchema + \
    getattr(NotifBase,'schema',Schema(())) + \
    schema

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class NotifNewsletter(NotifBase,BaseContent):
    """Newsletter"""
    security = ClassSecurityInfo()
    __implements__ = (getattr(NotifBase,'__implements__',()),) + (getattr(BaseContent,'__implements__',()),) + (INotifNewsletter,)

    receivers = []


    # This name appears in the 'add' box
    archetype_name             = 'Newsletter'

    meta_type                  = 'NotifNewsletter'
    portal_type                = 'NotifNewsletter'
    allowed_content_types      = []+NotifBase.allowed_content_types
    filter_content_types       = 1
    global_allow               = 0
    allow_discussion           = 0
    #content_icon               = 'Newsletter.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    suppl_views                = ()
    typeDescription            = "Newsletter"
    typeDescMsgId              = 'description_edit_newsletter'

    actions =  (


       {'action':      "string:${object_url}/notifnewsletter_send_form",
        'category':    "object",
        'id':          'send',
        'name':        'Send',
        'permissions': (CMFCorePermissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },


    )

    _at_rename_after_creation  = True

    schema = Newsletter_schema

    ##code-section class-header #fill in your manual code here
    # Because AGX don't take notice of parent actions
    actions =  NotifBase.actions + actions
    ##/code-section class-header


    #Methods
    #methods from Interface INewsletter

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,'send')
    def send(self):
        """Sends the newsletter
        """
        logger.info("Start of method send")

        # preparations
        request = self.REQUEST
        enl = self.aq_parent

        # get sender name
        sender_name = request.get("sender_name", "")
        if sender_name == "":
            sender_name = enl.getSenderName()

        # get sender e-mail
        sender_email = request.get("sender_email", "")
        if sender_email == "":
            sender_email = enl.getSenderEmail()

        # get test e-mail
        test_receiver = request.get("test_receiver", "")
        if test_receiver == "":
            test_receiver = enl.getTestEmail()

        # get subject
        subject = request.get("subject", "")
        if subject == "":
            subject = header_encode(self.Title(),charset='UTF-8')

        # Create from-header
        if enl.getSenderName():
            from_header = "%s <%s>" % (header_encode(sender_name,charset='UTF-8'), sender_email)
        else:
            from_header = sender_email

        # create multipart mail
        mail = MIMEMultipart("alternative")

        mail['From']    = from_header
        mail['Subject'] = subject
        mail.epilogue   = ''

        # exchange relative URLs
        parser = ENLHTMLParser(self)
        parser.feed(self.getText())
        text = parser.html

        # get charset
        props = getToolByName(self, "portal_properties").site_properties
        charset = props.getProperty("default_charset")

        # create & attach text part
        portal_transforms = getToolByName(self, 'portal_transforms')
        data = portal_transforms.convert('html_to_text', text)
        text_plain = data.getData()

        text_part = MIMEText(text_plain, "plain", charset)
        mail.attach(text_part)
        # create & attach html part with images

        html_part = MIMEMultipart("related")

        html_code = MIMEText(text, "html", charset)
        html_part.attach(html_code)

        # Add images to the message
        image_number = 0
        for image_url in parser.image_urls:
            image_url = urlparse(image_url)[2]
            o = self.restrictedTraverse(image_url)
            if hasattr(o, "_data"):                               # file-based
                image = MIMEImage(o._data)
            else:
                image = MIMEImage(o.data)                           # zodb-based
            image["Content-ID"] = "image_%s" % image_number
            image_number += 1
            html_part.attach(image)

        mail.attach(html_part)

        if hasattr(request, "test"):
            ##Send test email
            mail['To'] = test_receiver
            self.MailHost.send( mail.as_string() )
        else:
            ##Send an email for each subscriber
            self.receivers = self.getAllSubscribers()
            logger.info("Receivers: %s"%self.receivers)
            for receiver in self.receivers:
                if not(receiver is None or receiver == ''):
                    try:
                        self.MailHost.send( mail.as_string(),mto=receiver )
                        logger.info("Mail sended to: %s"%receiver)
                    except:
                        logger.info("Mail NOT sended to: %s"%receiver)

            # change status
            if not hasattr(request, "test"):
                wftool = getToolByName(self, "portal_workflow")
                wftool.doActionFor(self, "make sent")
            logger.info("Mail sended")
        logger.info("End of method send.")
    #manually created methods

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,"initializeArchetype")
    def initializeArchetype(self, **kwargs):
        """overwritten hook
        """
        NotifBase.initializeArchetype(self, **kwargs)
        #Get default acquireCriteria
        self.setAcquireCriteria(self.getParentNode().getAutomaticSend())
        #Get default template

        self.setDefaultTemplate(self.getParentNode().getDefaultTemplate())
        self.loadContent()



    security.declareProtected(CMFCorePermissions.ModifyPortalContent,"loadContent")
    def loadContent(self):
        """Load text dependend on criteria into text attribute
        """

        template = self.getNotifTemplate()
        newsletter_template = self.restrictedTraverse(template)
        newsletter_template.setNewsletter(self)
        text = newsletter_template.body()
        self.setText(text)


    security.declareProtected(CMFCorePermissions.View,"folder_contents")
    def folder_contents(self):
        """Overwritten to "forbid"" folder_contents
        """
        url = self.absolute_url()
        self.REQUEST.RESPONSE.redirect(url)

    security.declareProtected(CMFCorePermissions.View,"getAllSubscribers")
    def getAllSubscribers(self):
        """Returns a list of e-mail addresses from all subscribers
        """
        logger.info("Start of getAllSubscribers method")
        portal_catalog = getToolByName(self,'portal_catalog')
        query = {}
        query["portal_type"] = "NotifSubscriber"
        subscribers = portal_catalog.searchResults(**query)
        all_subscribers = [subscriber.getEmail for subscriber in subscribers]

        logger.info("Ends of getAllSubscribers method")
        return all_subscribers

    security.declareProtected(CMFCorePermissions.View,"getReceivers")
    def getReceivers(self):
        return self.receivers

    security.declareProtected(CMFCorePermissions.View, 'queryCatalog')
    def queryCatalog( self, REQUEST=None, **kw ):
        objects=NotifBase.queryCatalog(self,REQUEST, **kw )
        result=[]
        for item in objects:
            if item['portal_type'] != NotifNewsletter.portal_type:
                result.append(item)
        return result

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,'hasContentToSend')
    def hasContentToSend(self):
        logger.info("Start of hasContentToSend method")
        result=False
        try:
            result = len(self.synContentValues())>0
        except Exception,msg:
            import sys, traceback
            exc = sys.exc_info()
            logger.error(str(traceback.format_exception(exc[0],exc[1],exc[2])))
            del exc

        logger.info("End of hasContentToSend method")
        return result
registerType(NotifNewsletter,PROJECTNAME)
# end of class Newsletter

##code-section module-footer #fill in your manual code here
##/code-section module-footer



