# -*- coding: utf-8 -*-
#
# Notifica
# Copyright (C) 2007 GMV SGI Team <http://www.gmv-sgi.es>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# $Id: Notifica.py 615 2007-11-28 11:40:43Z frms $
# File: Notifica.py

__author__  = 'Fco. Jose Mulero - GMV'
__docformat__ = 'plaintext'

import logging

from DateTime import *
from zope import interface
from Products.Archetypes.public import DisplayList
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from NotifBase import NotifBase
from StringIO import StringIO
from Products.Notifica.interfaces.INotifica import INotifica
from Products.Notifica.config import PROJECTNAME,NOTIFICA_PATH,DEFAULT_ALARM
from Products.CMFCore import permissions as CMFCorePermissions
from Products.CMFPlone.utils import _createObjectByType
from Products.CMFPlone.PloneBatch import Batch
from NotifNewsletter import NotifNewsletter

from Products.Notifica.config import *
##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName
from zExceptions import BadRequest
import re

logger = logging.getLogger(PROJECTNAME)

ReferenceDates = DisplayList((
                    ('','')
                  , ('start', 'Start Date')
                  , ('end', 'End Date ')
                  , ('effective', 'Effective Date')
                  , ('created','Creation Date')
                  , ('modified','Modification Date')
    ))
schema=Schema((

    StringField('defaultTitle',
        widget=StringWidget(
            label="Default Title",
            description="Default for the title of the newsletters.",
            label_msgid='notifica_label_defaultTitle',
            description_msgid='notifica_help_defaultTitle',
            i18n_domain='notifica',
        ),
        required=True
    ),
    TextField('defaultDescription',
        widget=TextAreaWidget(
            label='Default Description',
            description="Default for the description of the newsletters.",
            label_msgid='notifica_label_defaultDescription',
            description_msgid='notifica_help_defaultDescription',
            i18n_domain='notifica',
        ),
    ),
    BooleanField('automaticSend',
        default="False",
        widget=BooleanWidget(
            label="Automatic Send",
            description = "If you want that newsletters sends automatically with the criteria defined in this level check this box.",
            label_msgid='notifica_label_automaticSend',
            description_msgid='notifica_help_automaticSend',
            i18n_domain='notifica',
        )
    ),
#===============================================================================
#    BooleanField('inheritCriteria',
#        default="True",
#        widget=BooleanWidget(
#            label="Inherit Criteria",
#            description="This field indicates that the Newsletters will inherit the criteria defined in this level.",
#            label_msgid='notifica_label_inheritCriteria',
#            description_msgid='notifica_help_inheritCriteria',
#            i18n_domain='notifica',
#        )
#    ),
#===============================================================================
    StringField('defaultTemplate',
        default='default',
        widget=SelectionWidget(
            macro="NotificaTemplateWidget",
            label="Default Template",
            description="Template, to generate the newsletter.",
            label_msgid='notifica_label_defaultTemplate',
            description_msgid='notifica_help_defaultTemplate',
            i18n_domain='notifica',
        ),
        required=True
    ),
    StringField('referenceDate',
        widget=SelectionWidget(
            label="Reference Date",
            label_msgid="notifica_label_referenceDate",
            description="Select the date criteria operation.",
            description_msgid="notifica_help_referenceDate",
            i18n_domain="notifica",
            format="select"),
        vocabulary=ReferenceDates

    ),
    IntegerField('alarm',
        widget=IntegerWidget(
            label="Reference Date",
            label_msgid="notifica_label_alarm",
            description="Days before end or start date when we want to send newsletters",
            description_msgid="notifica_help_alarm",
            i18n_domain="notifica",
         ),

    ),
#===============================================================================
#    StringField('senderEmail',
#        widget=StringWidget(
#            label="Sender e-mail",
#            description="Default for the sender address of the newsletters.",
#            label_msgid='notifica_label_senderEmail',
#            description_msgid='notifica_help_senderEmail',
#            i18n_domain='notifica',
#        ),
#        required=True,
#        validators=('isEmail',)
#    ),
#
#    StringField('senderName',
#        widget=StringWidget(
#            label="Sender name",
#            description="Default for the sender name of the newsletters.",
#            label_msgid='notifica_label_senderName',
#            description_msgid='notifica_help_senderName',
#            i18n_domain='notifica',
#        )
#    ),
#===============================================================================

    StringField('testEmail',
        widget=StringWidget(
            label="Test receiver",
            description="This email address will be used to send test email.",
            label_msgid='notifica_label_testreceiver',
            description_msgid='notifica_help_testreceiver',
            i18n_domain='notifica',
        ),
        validators=('isEmail',)
    ),



),
)

##/code-section module-header
##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

Notifica_schema = BaseFolderSchema + \
    getattr(NotifBase,'schema',Schema(()))+ \
    schema

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class Notifica(NotifBase,BaseFolder):
    """Object used for send newsletters with a content defined by the user."""
    security = ClassSecurityInfo()
    lastSend = DateTime('1999/1/1 00:00:00')
    future = DateTime('2012/12/31 23:59:59')
    referenceDate = 'start'
    idCriterion = ''
    # This name appears in the 'add' box
    archetype_name             = 'Notifica'

    meta_type                  = 'Notifica'
    portal_type                = 'Notifica'
    allowed_content_types      = ['NotifNewsletter', 'NotifTemplate', 'Image'] + NotifBase.allowed_content_types
    filter_content_types       = 1
    global_allow               = 1
    allow_discussion           = 0
    #content_icon               = 'Notifica.gif'
    immediate_view             = 'base_view'
    default_view               = 'base_view'

    suppl_views                = ()
    typeDescription            = "Object used for send newsletters with a content defined by the user. Please don't forget modify the criteria options all you need."
    typeDescMsgId              = 'description_edit_notifica'

    __implements__ = (getattr(NotifBase,'__implements__',()),) + (getattr(BaseFolder,'__implements__',()),) + (INotifica,)

    actions =  (


       {'action':      "string:${object_url}/view",
        'category':    "object",
        'id':          'view',
        'name':        'Archive',
        'permissions': (CMFCorePermissions.View,),
        'condition'  : 'python:1'
       },


       {'action':      "string:${object_url}/notifdrafts_view",
        'category':    "object",
        'id':          'drafts',
        'name':        'Drafts',
        'permissions': (CMFCorePermissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },

       {'action':      "string:${object_url}/notiftemplates_view",
        'category':    "object",
        'id':          'templates',
        'name':        'Templates',
        'permissions': (CMFCorePermissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },


    )

    _at_rename_after_creation  = True

    schema = Notifica_schema

    ##code-section class-header #fill in your manual code here
    # Because AGX don't take notice of parent actions
    actions =  NotifBase.actions + actions
    ##/code-section class-header


    #Methods
    #methods from Interface Notifica

    security.declarePublic("displayContentsTab")
    def displayContentsTab(self):
        """
        """
        return True

    security.declarePrivate("addCriterionDatePreSending")
    def addCriterionDatePreSending(self):
        """
        """
        if(str(self.idCriterion) != ''):
            try:
                self.deleteCriterion(str(self.idCriterion))
            except AttributeError:
                logger.info("The criterion %s, doesn't exists" %self.idCriterion)

        logger.info("%s"%self.getReferenceDate())
        if(self.getReferenceDate() != ''):
            date_crit = self.addCriterion(self.getReferenceDate(),'ATDateRangeCriterion')
            next = self.future
            try:
                next = DateTime() + 365
            except:
                logger.info("Max date exceeded")

            if(self.getReferenceDate() != 'end' and self.getReferenceDate() != 'start'):
                if date_crit.getStart() is None or date_crit.getStart().month is None or date_crit.getStart().day is None:
                    date_crit.setStart(self.lastSend)
                    date_crit.setEnd(next)
                    logger.info("Date crieterion: start %s end %s"%(date_crit.getStart(),date_crit.getEnd()))

            else:
                now = DateTime()
                if(self.getAlarm()is not None):
                    date_crit.setStart(now)
                    date_crit.setEnd(now+self.getAlarm())
                    logger.info("Date crieterion: start %s end %s"%(date_crit.getStart(),date_crit.getEnd()))
                else:
                    date_crit.setStart(now)
                    date_crit.setEnd(now+DEFAULT_ALARM)
                    logger.info("Date crieterion: start %s end %s"%(date_crit.getStart(),date_crit.getEnd()))

            self.idCriterion = date_crit.id

    security.declarePrivate("addCriterionDatePostSending")
    def addCriterionDatePostSending(self):
        """
        """
        if self.getReferenceDate() != 'end' and self.getReferenceDate() != 'start':
            if(str(self.idCriterion) != ''):
                try:
                    self.deleteCriterion(str(self.idCriterion))
                except AttributeError:
                    logger.info("The criterion %s, doesn't exists" %self.idCriterion)

            logger.info("%s"%self.getReferenceDate())
            if(self.getReferenceDate() != ''):
                date_crit = self.addCriterion(self.getReferenceDate(),'ATDateRangeCriterion')
                next = self.future
                try:
                    next = DateTime() + 365
                except:
                    logger.info("Max date exceeded")

                if(self.getReferenceDate() != 'end' and self.getReferenceDate() != 'start'):
                    date_crit.setStart(self.lastSend)
                    date_crit.setEnd(next)


                self.idCriterion = date_crit.id
                logger.info("Method post sending: start date %s end date %s" % (date_crit.getStart(),date_crit.getEnd()))

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,"sendNewsletter")
    def sendNewsletter(self):
        plone_tool = getToolByName(self, 'plone_utils')

        logger.info("Start of sendNewsletter method")
        container = self
        hora = DateTime()
        title ="%s %s" %( self.getDefaultTitle() , hora.strftime('%d %b %Y %H:%M'))
        id = plone_tool.normalizeString(title)

        self.addCriterionDatePreSending()
        container.invokeFactory('NotifNewsletter',id=id)
        newsletter = getattr(container,id)


        if(newsletter.hasContentToSend() and len(newsletter.getAllSubscribers())>0):
            logger.info("There are newsletters to send")
            try:
                newsletter.setTitle(self.getDefaultTitle())
                newsletter.setDescription(self.getDefaultDescription())
                newsletter.setCreators(self.creators)
                newsletter.send()

                self.lastSend = DateTime()
                self.addCriterionDatePostSending()
            except:
                container.manage_delObjects([id])
        else:
            logger.info("There are not newsletters to send")
            container.manage_delObjects([id])

        logger.info("End of sendNewsletter method")

    security.declarePrivate("at_post_edit_script")
    def at_post_edit_script (self):
        self.addCriterionDatePreSending()

    security.declarePrivate("at_post_create_script")
    def at_post_create_script (self):
        self.addCriterionDatePreSending()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,"initializeArchetype")
    def initializeArchetype(self, **kwargs):
        """overwritten hook
        """
        NotifBase.initializeArchetype(self, **kwargs)
        self.title = self.portal_type
        # Add default template
        templateTitle = self.translate("Default Template",domain='notifica')
        templateDescription = self.translate("Default Template Description",domain='notifica')
        self.manage_addProduct["Notifica"].addNotifTemplate(id="default", title=templateTitle,description=templateDescription)

        tmpl = self.default


    security.declarePublic('queryNewsletters')
    def queryNewsletters(self,REQUEST=None):
        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', {})

        b_start = REQUEST.get('b_start', 0)

        q={}
        q['portal_type']='NotifNewsletter'
        q['sort_on']='created'
        q['sort_order']='reverse'
        q['path']='/'.join(self.getPhysicalPath())

        pcatalog = getToolByName(self, 'portal_catalog')
        limit = self.getLimitNumber()
        max_items = self.getItemCount()
        # Batch based on limit size if b_szie is unspecified

        b_size = 20
        results = pcatalog.searchResults(REQUEST, **q)

        batch = Batch(results, b_size, int(b_start), orphan=0)
        return batch

    security.declareProtected(CMFCorePermissions.View,'getSenderEmail')
    def getSenderEmail(self):
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()
        return portal.getProperty('email_from_address')

    security.declareProtected(CMFCorePermissions.View,'getSenderName')
    def getSenderName(self):
        urltool = getToolByName(self, 'portal_url')
        portal = urltool.getPortalObject()
        return portal.getProperty('email_from_name')

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'post_validate')
    def post_validate(self, REQUEST=None, errors=None):
        """Validates if automaticSend is set then the referenceDate doesn't must be empty
        """
        NotifBase.post_validate(self,REQUEST,errors)
        BaseFolder.post_validate(self,REQUEST,errors)

        if 'referenceData' in errors:
            # No point in validating bad input
            return

        referenceDate = REQUEST.get('referenceDate', None)
        automaticSend = REQUEST.get('automaticSend',None)

        if automaticSend =='1' and not referenceDate:
            errors['referenceDate'] = 'This field is required, please provide some information'

    security.declarePrivate("synContentValues")
    def synContentValues(self):
        """Method to sindicate content"""
        contents = self.getFolderContents();
        result = []
        for content in contents:
            if content.portal_type == NotifNewsletter.portal_type:
                result.append(content)
        return result
registerType(Notifica,PROJECTNAME)
# end of class Notifica

##code-section module-footer #fill in your manual code here
##/code-section module-footer
